import React, { useState } from 'react';
import Member from './Member';
import logo from '../../logo.svg';
import Mech from '../../Mech';
import MechObserver from '../../MechObserver';
import { isEmptySlot } from '../../MechUtils';

function instead<T>(array: T[], index: number, newItem: T): T[] {
    return array.map((item, i) => {
        if (i === index) {
            return newItem;
        } else {
            return item;
        }
    });
}

function hasFreeSlot(squad: Mech[]) {
    return squad.some(isEmptySlot)
}

function isSquadFull(squad: Mech[]) {
    return !hasFreeSlot(squad);
}


function Squad({pickObservers} : {pickObservers: {[key: string] : MechObserver}}) {

    const placeHolder: Mech = {id: -1, name: 'Empty slot', kind: '_', health: 0};
    const [squadMechs, setSquadMechs] = useState([placeHolder, placeHolder, placeHolder]);

    function putInFreeSlot(mech: Mech, squad: Mech[]): Mech[] {
        const freeSlotIndex = squad.findIndex(isEmptySlot);
        console.log('Next free slot : ', freeSlotIndex);
        return instead(squad, freeSlotIndex, mech);
    }


    function addToSquad(mech: Mech) {
        console.log("New mech to add :", mech)
        if (isSquadFull(squadMechs)) {
            console.log("Squad is full, sorry");
            return;
        }

        setSquadMechs(putInFreeSlot(mech, squadMechs));
    }

    function removeFromSquad(index: number) {
        setSquadMechs(instead(squadMechs, index, placeHolder));
    }

    pickObservers.addWhenPicked = addToSquad;

    return (
        <article className='custom-squad'>
            <h2>Custom squad</h2>
            <div className='mech-members'>
                {squadMechs.map((m, index) => (
                    <Member mech={m} remover={() => removeFromSquad(index)}/>
                ))}
            </div>
            <div className='squad-description'>
                <label htmlFor='squad-description-field-id'>Squad description</label>
                <textarea id='squad-description-field-id'>
                    Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.
                </textarea>
            </div>
        </article>
    )
}

export default Squad;